"""
This submodule implements an OO abstraction for the query language.

Example:

cond('packageName', 'com.whatsapp') & cond('origin', 'GooglePlay')
=> "packageName:com.whatsapp AND origin:GooglePlay"
"""

from functools import reduce

class query:
    """
    Common interface for indentifing an OO query during typechecking.
    """
    def __or__(self, other):
        return _or(self, other)
        
    def __and__(self, other):
        return _and(self, other)

    def __neg__(self):
        return _not(self)

class query_group(query):
    def __init__(self, elems, join_op):
        self.elems = elems
        self.join_op = join_op

    def __str__(self):
        def f(e):
            if e is query_group:
                return "(%s)" % str(e)
            else:
                return str(e)
        return " {} ".format(self.join_op).join(map(f, self.elems))

class _or(query_group):
    """
    Represents an OR combination between elements in a query.
    """
    def __init__(self, left, right):
        if isinstance(left, list):
            if len(left) > 2:
                left = left[0] | left[1:]
            else:
                left = left[0] | left[1]
        if isinstance(right, list):
            if len(right) > 2:
                right = right[0] | right[1:]
            else:
                right = right[0] | right[1]
        super().__init__([left, right], 'OR')                

class _and(query_group):
    """
    Represents an AND combination between elements in a query.
    """
    def __init__(self, left, right):
        if isinstance(left, list):
            if len(left) > 2:
                left = left[0] & left[1:]
            else:
                left = left[0] & left[1]
        if isinstance(right, list):
            if len(right) > 2:
                right = right[0] & right[1:]
            else:
                right = right[0] & right[1]
        super().__init__([left, right], 'AND')

class _not(query):
    """
    Represents a negative query.
    """
    def __init__(self, inner):
        self.inner = inner

    def __neg__(self):
        return self.inner

    def __str__(self):
        return "-{}".format(self.inner)

class cond(query):
    """
    Represents a cond inside the query.
    If you want to create a fixed filter you should inherit from this class.
    """
    def __init__(self, field, condition, literal=False):
        self.field = field
        self.condition = condition
        self.literal = literal

    def __str__(self):
        if self.literal:
            fmt = "{field}:\"{condition}\""
        else:
            fmt = "{field}:{condition}"
        return fmt.format(field=self.field, condition=self.condition)

class packageName(cond):
    """
    Filter for package names of applications
    """
    def __init__(self, pkgName, literal=False):
        super().__init__('packageName', pkgName, literal)

fromGooglePlay = cond('origin', 'GooglePlay')
fromAppStore = cond('origin', 'AppleStore')

def or_many(*lst):
    return reduce(lambda x, y: x | y, lst)

def and_many(*lst):
    return reduce(lambda x, y: x & y, lst)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import logging
import time
import json
import hashlib
from .Response import Response
from ..Version import Version
from . import Error

logger = logging.getLogger('tacyt.auth')


class Auth(object):
    API_PORT = 443
    API_HTTPS = True
    API_PROXY = None
    API_PROXY_PORT = None

    AUTHORIZATION_METHOD = "11PATHS"
    X_11PATHS_HEADER_PREFIX = "X-11paths-"
    BODY_HASH_HEADER_NAME = X_11PATHS_HEADER_PREFIX + "Body-Hash"
    AUTHORIZATION_HEADER_FIELD_SEPARATOR = " "
    UTC_STRING_FORMAT = "%Y-%m-%d %H:%M:%S"
    CHARSET_ASCII = "US-ASCII"
    CHARSET_ISO_8859_1 = "ISO-8859-1"
    CHARSET_UTF_8 = "UTF-8"
    HTTP_METHOD_GET = "GET"
    HTTP_METHOD_POST = "POST"
    HTTP_METHOD_PUT = "PUT"
    HTTP_METHOD_DELETE = "DELETE"
    HTTP_HEADER_CONTENT_LENGTH = "Content-Length"
    HTTP_HEADER_CONTENT_TYPE = "Content-Type"
    HTTP_HEADER_CONTENT_TYPE_FORM_URLENCODED = "application/x-www-form-urlencoded"
    HTTP_HEADER_CONTENT_TYPE_JSON = "application/json"
    PARAM_SEPARATOR = "&"
    PARAM_VALUE_SEPARATOR = "="
    X_11PATHS_HEADER_SEPARATOR = ":"
    FILE_HASH_HEADER_NAME = X_11PATHS_HEADER_PREFIX + "File-Hash"
    AUTHORIZATION_HEADER_NAME = "Authorization"
    DATE_HEADER_NAME = X_11PATHS_HEADER_PREFIX + "Date"
    MULTIPART_FORM_DATA = "multipart/form-data"
    THREAD_POOL_SIZE = 10

    def __init__(self, appId, secretKey):
        '''
        Create an instance of the class with the Application ID and secret obtained from Tacyt
        @param $appId
        @param $secretKey
        '''
        self.appId = appId
        self.secretKey = secretKey

    @staticmethod
    def set_host(host):
        '''
        @param $host The host to be connected with (http://hostname) or (https://hostname)
        '''
        if host.startswith("http://"):

            Version.API_HOST = host[len("http://"):]
            Auth.API_PORT = 80
            Auth.API_HTTPS = False

        elif host.startswith("https://"):
            Version.API_HOST = host[len("https://"):]
            Auth.API_PORT = 443
            Auth.API_HTTPS = True

    @staticmethod
    def set_proxy(proxy, port):
        '''
        Enable using a Proxy to connect through
        @param $proxy The proxy server
        @param $port The proxy port number
        '''
        Auth.API_PROXY = proxy
        Auth.API_PROXY_PORT = port

    @staticmethod
    def get_part_from_header(part, header):
        '''
        The custom header consists of three parts, the method, the appId and the signature.
        This method returns the specified part if it exists.
        @param $part The zero indexed part to be returned
        @param $header The HTTP header value from which to extract the part
        @return string the specified part from the header or an empty string if not existent
        '''
        if header:
            parts = header.split(Auth.AUTHORIZATION_HEADER_FIELD_SEPARATOR)
            if len(parts) >= part:
                return parts[part]
        return ""

    @staticmethod
    def get_auth_method_from_header(authorization_header):
        '''
        @param $authorization_header authorization HTTP Header
        @return string the authorization method. Typical values are "Basic", "Digest" or "11PATHS"
        '''
        return Auth.get_part_from_header(0, authorization_header)

    @staticmethod
    def parse_query_params(query_params):
        global query, key, value
        if query_params is None or len(query_params) == 0:
            return ""

        if len(query_params) > 0:
            query = "?"
        else:
            query = ""

        for elements in query_params:
            key = query_params['key']
            value = query_params['value']
            if len(value) > 0:
                query += key + Auth.PARAM_VALUE_SEPARATOR + value.encode(Auth.CHARSET_UTF_8) + Auth.PARAM_SEPARATOR

        if len(query) > 0 and query[len(query) - 1] == '&':

            return query[0:(len(query) - 1)]

        else:
            return query

    def get_api_host(self):
        return Version.API_HOST

    def http_get_proxy(self, url, query_params=None):
        try:
            query = Auth.parse_query_params(query_params)
            url += query
            return self.http_get(self.get_api_host() + url,
                                 self.authentication_headers(self.HTTP_METHOD_GET, url, None, None))

        except:
            logger.exception("Failed to make get request")
            return None

    def http_get(self, url, headers):
        return self._http(Auth.HTTP_METHOD_GET, url, headers, None)

    def http_delete_proxy(self, url):
        try:
            return self.http_delete(self.get_api_host() + url,
                                    self.authentication_headers(self.HTTP_METHOD_DELETE, url, None), None)

        except:
            logger.exception("Failed to make delete request")
            return None

    def http_post_proxy(self, url, data=None, body=None):
        try:
            if data is not None:
                return self.http_post(self.get_api_host() + url,
                                      self.authentication_headers(self.HTTP_METHOD_POST, url, None, data), data)

            elif body is not None:
                return self.http_post(self.get_api_host() + url,
                                      self.authentication_headers_with_body(self.HTTP_METHOD_POST, url, None, body),
                                      body)

        except:
            logger.exception("Failed to make post request")
            return None

    def http_put_proxy(self, url, data=None, body=None):
        try:
            if data is not None:
                return self.http_put(self.get_api_host() + url,
                                     self.authentication_headers(self.HTTP_METHOD_PUT, url, None, data), data)

            elif body is not None:
                return self.http_put(self.get_api_host() + url,
                                     self.authentication_headers_with_body(self.HTTP_METHOD_PUT, url, None, body), body)
        except:
            logger.exception("Failed to make put request")
            return None

    @staticmethod
    def get_appId_from_header(authorization_header):
        '''
        @param $authorization_header authorization HTTP Header
        @return string the requesting application Id. Identifies the application using the API
        '''
        return Auth.get_part_from_header(1, authorization_header)

    @staticmethod
    def get_signature_from_header(authorization_header):
        '''
        @param $authorization_header authorization HTTP Header
        @return string the signature of the current request. Verifies the identity of the application using the API
        '''
        return Auth.get_part_from_header(2, authorization_header)

    @staticmethod
    def get_current_UTC():
        '''
        @return a string representation of the current time in UTC to be used in a Date HTTP Header
        '''
        return time.strftime(Auth.UTC_STRING_FORMAT, time.gmtime())

    def _http(self, method, url, x_headers=None, body=None, file=None, content_type=None):
        '''
        HTTP Request to the specified API endpoint
        @param method string
        @param x_headers list
        @param body dict json
        @return TacytResponse
        '''

        auth_headers = None
        json_body = None

        try:
            # Try to use the new Python3 HTTP library if available
            import http.client as http
            import urllib.parse as urllib
        except:
            # Must be using Python2 so use the appropriate library
            import httplib as http
            import urllib

        if Auth.API_PROXY != None:
            if Auth.API_HTTPS:
                conn = http.HTTPSConnection(Auth.API_PROXY, Auth.API_PROXY_PORT)
                conn.set_tunnel(Version.API_HOST, Auth.API_PORT)
            else:
                conn = http.HTTPConnection(Auth.API_PROXY, Auth.API_PROXY_PORT)
                url = "http://" + Version.API_HOST + url
        else:
            if Auth.API_HTTPS:
                conn = http.HTTPSConnection(Version.API_HOST, Auth.API_PORT)
            else:
                conn = http.HTTPConnection(Version.API_HOST, Auth.API_PORT)
        try:
            if self.HTTP_METHOD_GET == method or self.HTTP_METHOD_DELETE == method:
                auth_headers = self.authentication_headers(method, url, x_headers, None, None)
                conn.request(method, url, headers=auth_headers)
            elif self.HTTP_METHOD_POST == method or self.HTTP_METHOD_PUT == method:
                if body is not None and file is None:
                    json_body = json.dumps(body)
                    auth_headers = self.authentication_headers_with_body(method, url, x_headers, json_body, None)
                    auth_headers[self.HTTP_HEADER_CONTENT_TYPE] = content_type

            all_headers = auth_headers

            if body is not None:
                conn.request(method=method, url=url, body=json_body, headers=all_headers)

            res = conn.getresponse()
            response_data = res.read().decode('utf8')

            conn.close()
            ret = Response(json_string=response_data)

        except Exception as e:
            logger.exception("Failed to make http request with body {}".format(body))
            ret = None

        return ret

    def sign_data(self, data):
        '''
        @param $data the string to sign
        @return string base64 encoding of the HMAC-SHA1 hash of the data parameter using {@code secretKey} as cipher key.
        '''
        from hashlib import sha1
        import hmac
        import binascii

        sha1_hash = hmac.new(self.secretKey.encode(), data.encode(), sha1)
        return binascii.b2a_base64(sha1_hash.digest())[:-1].decode('utf8')

    def authentication_headers(self, http_method, query_string, x_headers=None, utc=None, params=None):
        '''
        Calculate the authentication headers to be sent with a request to the API
        @param $http_method the HTTP Method
        @param $query_string the urlencoded string including the path (from the first forward slash) and the parameters
        @param $x_headers HTTP headers specific to the 11-paths API. null if not needed.
        @param $utc the Universal Coordinated Time for the Date HTTP header
        @return array a map with the authorization and Date headers needed to sign a Tacyt API request
        '''

        if not utc:
            utc = Auth.get_current_UTC()

        utc = utc.strip()

        string_to_sign = (http_method.upper().strip() + "\n" +
                          utc + "\n" +
                          self.get_serialized_headers(x_headers) + "\n" +
                          query_string.strip())

        if params is not None and len(params) > 0:
            serialized_params = self.get_serialized_params(params)
            if serialized_params is not None and len(serialized_params) > 0:
                string_to_sign = string_to_sign + "\n" + serialized_params

        authorization_header = (Auth.AUTHORIZATION_METHOD + Auth.AUTHORIZATION_HEADER_FIELD_SEPARATOR +
                                self.appId + Auth.AUTHORIZATION_HEADER_FIELD_SEPARATOR +
                                self.sign_data(string_to_sign))

        headers = dict()
        headers[Auth.AUTHORIZATION_HEADER_NAME] = authorization_header
        headers[Auth.DATE_HEADER_NAME] = utc

        return headers

    def authentication_headers_with_body(self, http_method, query_string, x_headers=None, body=None, utc=None):
        """
        Calculates the headers to be sent with a request to the API so the server
            can verify the signature
        @param method The HTTP request method.
        @param queryString The urlencoded string including the path (from the
            first forward slash) and the parameters.
        @param xHeaders The HTTP request headers specific to the API, excluding
            X-11Paths-Date. null if not needed.
        @param body The HTTP request body. Null if not needed.
        @param utc the Universal Coordinated Time for the X-11Paths-Date HTTP header
        @return header with the {@value #AUTHORIZATION_HEADER_NAME}, the {@value
            #DATE_HEADER_NAME} and the {@value #BODY_HASH_HEADER_NAME} headers
            needed to be sent with a request to the API.
        """

        headers = dict()
        body_hash = None

        if body is not None:
            body_hash = hashlib.sha1(str(body).encode('utf-8')).hexdigest()
            if x_headers is None:
                x_headers = dict()
            else:
                for key, value in x_headers.items():
                    headers[key] = value

            x_headers[Auth.BODY_HASH_HEADER_NAME] = body_hash

        if not utc:
            utc = Auth.get_current_UTC()

        utc = utc.strip()

        string_to_sign = (http_method.upper().strip() + "\n" +
                          utc + "\n" +
                          self.get_serialized_headers(x_headers) + "\n" +
                          query_string.strip())

        authorization_header = (Auth.AUTHORIZATION_METHOD + Auth.AUTHORIZATION_HEADER_FIELD_SEPARATOR +
                                self.appId + Auth.AUTHORIZATION_HEADER_FIELD_SEPARATOR +
                                self.sign_data(string_to_sign))

        headers[Auth.AUTHORIZATION_HEADER_NAME] = authorization_header
        headers[Auth.DATE_HEADER_NAME] = utc
        if body_hash is not None:
            headers[Auth.BODY_HASH_HEADER_NAME] = body_hash

        return headers

    def get_serialized_headers(self, x_headers):
        '''
        Prepares and returns a string ready to be signed from the 11-paths specific HTTP headers received
        @param $x_headers a non neccesarily ordered map (array without duplicates) of the HTTP headers to be ordered.
        @return string The serialized headers, an empty string if no headers are passed, or None if there's a problem such as non 11paths specific headers
        '''
        if x_headers:
            headers = dict((k.lower(), v) for k, v in x_headers.items())

            serialized_headers = ""
            for key, value in sorted(headers.items()):
                if not key.startswith(Auth.X_11PATHS_HEADER_PREFIX.lower()):
                    logging.error(
                        "Error serializing headers. Only specific " + Auth.X_11PATHS_HEADER_PREFIX + " headers need to be signed")
                    return None
                serialized_headers += key + Auth.X_11PATHS_HEADER_SEPARATOR + value + " "
            return serialized_headers.strip()
        else:
            return ""

    def get_serialized_params(self, params):
        """
         Prepares and returns a string ready to be signed from the params of an HTTP request
         The params must be only those included in the body of the HTTP request
            when its content type is application/x-www-urlencoded and must beurldecoded.
         @param params The params of an HTTP request.
         @return A serialized representation of the params ready to be signed.
            null if there are no valid params.
        """
        try:
            # Try to use the new Python3 HTTP library if available
            import http.client as http
            import urllib.parse as urllib
        except:
            # Must be using Python2 so use the appropriate library
            import httplib as http
            import urllib
        if params:
            serialized_params = ""
            for key in sorted(params):
                serialized_params += key + Auth.PARAM_VALUE_SEPARATOR + urllib.quote_plus(
                    params[key]) + Auth.PARAM_SEPARATOR
            return serialized_params.strip(Auth.PARAM_SEPARATOR)
        else:
            return ""

    def http_delete(self, url, headers, body):
        return self._http(self.HTTP_METHOD_DELETE, url, headers, body)

    def http_post(self, url, headers, data=None, body=None):
        if data is not None:
            return self._http(self.HTTP_METHOD_POST, url, headers, data)
        elif body:
            return self._http(self.HTTP_METHOD_POST, url, headers, body, None, self.HTTP_HEADER_CONTENT_TYPE_JSON)
        raise Exception("data and body can't be both None")

    def http_post_file(self, url, headers, file_stream, file_name, tag_name):
        try:
            import requests
        except Exception:
            return Response(error=Error.Error({"code": "-1",
                                               "message": "The Python \"requests\" library was not found. Please, install it before call the upload method."}))

        files = {'file': (file_name, file_stream, 'application/octet-stream')}
        if Auth.API_HTTPS:
            url = "https://" + self.get_api_host() + url
        else:
            url = "http://" + self.get_api_host() + url

        if self.API_PROXY is not None:
            proxy_host = "%s:%d" % (self.API_PROXY, self.API_PROXY_PORT)
            proxies = {"http": proxy_host, "https": proxy_host}
        else:
            proxies = None

        if tag_name is None:
            res = requests.post(url, headers=headers, files=files, proxies=proxies)
        else:
            res = requests.post(url, headers=headers, files=files, data={'tagName': tag_name}, proxies=proxies)
        res.raise_for_status()

        response_data = Response(json_string=res.content)

        return response_data

    def http_put(self, url, headers, data=None, body=None):
        if data is not None:
            return self._http(self.HTTP_METHOD_PUT, url, headers, data)
        elif body is not None:
            return self._http(self.HTTP_METHOD_PUT, url, headers, body, None, self.HTTP_HEADER_CONTENT_TYPE_JSON)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
This library offers an API to use Tacyt in a python environment.
Copyright (C) 2015 Eleven Paths
'''
try:
    import simplejson as json

except ImportError:
    import json

json_encode = lambda x: json.dumps(x)
json_decode = lambda x: json.loads(x)

class ExternalApiTagRequest:

    LIST_REQUEST = "LIST"
    CREATE_REQUEST = "CREATE"
    REMOVE_REQUEST = "REMOVE"
    REMOVE_ALL_REQUEST = "REMOVE_ALL"

    request_type = None
    tag = None
    apps = []

    def __init__(self, request_type=None, tag=None, apps=None):
        self.request_type = request_type
        self.tag = tag
        self.apps = apps

    def get_json_encode_dict_for_tag_based_requests(self):
        json_obj = dict()
        if self.request_type is not None:
            json_obj["requestType"] = self.request_type
        if self.tag is not None:
            json_obj["tag"] = self.tag
        if self.apps is not None:
            json_obj["apps"] = self.apps

        return json_obj

    def get_json_encode_string(self):

        return json_encode(self.get_json_encode_dict_for_tag_based_requests())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
This library offers an API to use Tacyt in a python environment.
Copyright (C) 2015 Eleven Paths
'''

import json

from . import Error


class Response(object):
    '''
    This class models a response from any of the endpoints in the Tacyt API.
    It consists of a "data" and an "error" elements. Although normally only one of them will be
    present, they are not mutually exclusive, since errors can be non fatal, and therefore a response
    could have valid information in the data field and at the same time inform of an error.
    '''

    def __init__(self, data=None, error=None, json_string=None ):
        '''
        @param $json a json string received from one of the methods of the Tacyt API
        '''
        self.data = data
        self.error = error
        if json_string is not None:
            json_object = json.loads(json_string)
            if "data" in json_object:
                self.data = json_object["data"]
            else:
                self.data = ""
            if "error" in json_object:
                self.error = Error.Error(json_object["error"])
            else:
                self.error = ""

    def get_data(self):
        '''
        @return JsonObject the data part of the API response
        '''
        return self.data

    def set_data(self, data):
        '''
        @param $data the data to include in the API response
        '''
        self.data = json.loads(data)

    def get_error(self):
        '''
        @return Error the error part of the API response, consisting of an error code and an error message
        '''
        return self.error

    def set_error(self, error):
        '''
        @param $error an error to include in the API response
        '''
        self.error = Error.Error(error)

    def to_json(self):
        '''
        @return a Json object with the data and error parts set if they exist
        '''
        json_response = {}

        if hasattr(self, "data"):
            json_response["data"] = self.data

        if hasattr(self, "error"):
            json_response["error"] = self.error

        return json_response



#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
This library offers an API to use Tacyt in a python environment.
Copyright (C) 2015 Eleven Paths
'''


class ExternalApiCompareRequest:

    FIELD_MATCHING_FIELDS = "matchingFields"
    FIELD_MATCHING_QUERY = "matchingQuery"
    FIELD_APPS = "apps"

    apps = []
    include_details = None

    def __init__(self, apps, include_details):
        self.apps = apps
        self.include_details = include_details

    def get_json_encode_for_compare_apps(self):
        json_data = dict()
        if self.apps is not None:
            json_data["apps"] = self.apps
        if self.include_details is not None:
            json_data["includeDetails"] = self.include_details

        return json_data



#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
 This library offers an API to use Tacyt in a python environment.
 Copyright (C) 2015 Eleven Paths
'''

class Filter():

    class Rule():

        weight = None
        content = None

        def __init__(self, weight=None, content=None):
            self.weight = weight
            self.content = content

        def get_json_encode(self):
            return {"weight": self.weight, "content": self.content}

    PUBLIC_VISIBILITY = "PUBLIC"
    PRIVATE_VISIBILITY = "PRIVATE"

    id = None
    name = None
    description = None
    weight = None
    visibility = PRIVATE_VISIBILITY
    rules = [Rule(None, None)]
    groups = None

    def __init__(self, id=None, name=None, description=None, weight=None, visibility=None, rules=None, groups=None):
        self.id = id
        self.name = name
        self.description = description
        self.weight = weight
        self.visibility = visibility
        self.rules = rules
        self.groups = groups

    def get_json_encode(self):

        result = {"id": self.id,
                  "name": self.name,
                  "description": self.description,
                  "weight": self.weight,
                  "visibility": self.visibility,
                  "rules": self.rules,
                  "groups": self.groups}

        filter_rules = list()

        if self.rules is not None:
            for rule in self.rules:
                filter_rules.append(rule.get_json_encode())

            result["rules"] = filter_rules

        return result


#!/usr/bin/env python
# -*- coding: utf-8 -*-
import logging

from .ExternalApiFilterRequest import ExternalApiFilterRequest
from .ExternalApiTagRequest import ExternalApiTagRequest
from .Filter import Filter
from .ExternalApiCompareRequest import ExternalApiCompareRequest
from .ExternalApiSearchRequest import ExternalApiSearchRequest
from .Version import Version
from .authorization.Auth import Auth

import hashlib
from os import path

logger = logging.getLogger('tacyt.app')

try:
    import simplejson as json
except ImportError:
    import json


def json_encode(x): return json.dumps(x)


def json_decode(x): return json.loads(x)


class TacytApp(Auth):
    API_SEARCH_URL = "/api/" + Version.API_VERSION + "/search"
    API_DETAILS_URL = "/api/" + Version.API_VERSION + "/details"
    API_FILTERS_URL = "/api/" + Version.API_VERSION + "/filters"
    API_TAGS_URL = "/api/" + Version.API_VERSION + "/tags"
    API_COMPARER_URL = "/api/" + Version.API_VERSION + "/compare"
    API_UPLOAD_URL = "/api/" + Version.API_VERSION + "/upload"

    def __init__(self, app_id, secret_key):
        """
        Create an instance of the class with the Application ID and secret obtained from Tacyt
        @param $app_id
        @param $secret_key
        """
        super(TacytApp, self).__init__(app_id, secret_key)

    def search_apps(self, query, numberPage=None, maxResults=None, outfields=None, grouped=None):
        """
        @param $query The query string will filter the search results.
        @param $numberPage A number greater or equal to 1 indicating the page of results which have to be retrieved.
        @param $maxResults A number between 1 and 100 indicating the max number of apps which have to be retrieved.
        @return Json structure with the keys to the Applications found.
        """
        result = ExternalApiSearchRequest(query, numberPage, maxResults, outfields, grouped)
        return self.http_post(self.API_SEARCH_URL, None, body=result.get_json_encode_for_search())

    def get_app_details(self, key):
        """
        @param $key The key of an application.
        @return Json structure with the details of an application.
        """
        return self.http_get(self.API_DETAILS_URL + "/" + key, None)

    def list_tags(self):
        """
        @return A list of tags that have been created.
        """
        result = ExternalApiTagRequest(ExternalApiTagRequest.LIST_REQUEST, None, None)
        return self.http_post(self.API_TAGS_URL, None, body=result.get_json_encode_dict_for_tag_based_requests())

    def assign_tag(self, tag, app_keys):
        """
        This method associates a tag created with applications.
        @param $tag the name of the tag to create
        @param $app_keys Key applications that want to associate with the tag
        @return A list of applications associates with a tag.
        """
        result = ExternalApiTagRequest(ExternalApiTagRequest.CREATE_REQUEST, tag, app_keys)
        return self.http_post(self.API_TAGS_URL, None, body=result.get_json_encode_dict_for_tag_based_requests())

    def remove_tag_for_apps(self, tag, app_keys):
        """
        This method remove a tag associate with applications.
        @param $tag the name of the tag to create
        @param $app_keys Key applications that want to remove with the tag
        """
        result = ExternalApiTagRequest(ExternalApiTagRequest.REMOVE_REQUEST, tag, app_keys)
        return self.http_post(self.API_TAGS_URL, None, body=result.get_json_encode_dict_for_tag_based_requests())

    def delete_tag(self, tag):
        """
        This method delete a tag.
        @param $tag the name of the tag you want to delete.
        """
        result = ExternalApiTagRequest(ExternalApiTagRequest.REMOVE_ALL_REQUEST, tag, None)
        return self.http_post(self.API_TAGS_URL, None, body=result.get_json_encode_dict_for_tag_based_requests())

    def create_filter(self, filter):
        """
        This method create a filter.
        @param $filter Filter structure.
        """
        result = ExternalApiFilterRequest(ExternalApiFilterRequest.CREATE_REQUEST, filter, 0, None)
        return self.http_post(self.API_FILTERS_URL, None, body=result.get_json_encode_for_filter_based_requests())

    def update_filter(self, filter):
        """
        This method update changes associates with a filter.
        @param $filter Filter structure.
        """
        result = ExternalApiFilterRequest(ExternalApiFilterRequest.UPDATE_REQUEST, filter, 0, None)
        return self.http_post(self.API_FILTERS_URL, None, body=result.get_json_encode_for_filter_based_requests())

    def read_group_filters(self):
        """
        @return all group filters created
        """
        result = ExternalApiFilterRequest(ExternalApiFilterRequest.READ_GROUPS, None, 0, None)
        return self.http_post(self.API_FILTERS_URL, None, body=result.get_json_encode_for_filter_based_requests())

    def read_all_filters(self):
        """
        @return a list of filters creates.
        """
        result = ExternalApiFilterRequest(ExternalApiFilterRequest.READ_REQUEST, None, 0, None)
        return self.http_post(self.API_FILTERS_URL, None, body=result.get_json_encode_for_filter_based_requests())

    def read_one_filter(self, filter_id):
        """
        @param $filter_id id of the filter you want to read.
        @return This method returns the details of filter associate with this filter_id.
        """
        filter = Filter(filter_id)
        result = ExternalApiFilterRequest(ExternalApiFilterRequest.READ_REQUEST, filter, 0, None)
        return self.http_post(self.API_FILTERS_URL, None, body=result.get_json_encode_for_filter_based_requests())

    def delete_filter(self, filter_id):
        """
        This method delete a filter create.
        @param $filter_id id of the filter you want to delete.
        """
        filter = Filter(filter_id)
        result = ExternalApiFilterRequest(ExternalApiFilterRequest.DELETE_REQUEST, filter, 0, None)
        return self.http_post(self.API_FILTERS_URL, None, body=result.get_json_encode_for_filter_based_requests())

    def search_public_filter(self, query, page):
        """
        @param $query any word or phrase within the description or title Filter
        @param $page A number greater or equal to 1 indicating the page of results which have to be retrieved.
        @return A list of public filters(Visibility = Public)
        """
        result = ExternalApiFilterRequest(ExternalApiFilterRequest.SEARCH_PUBLIC_FILTER_REQUEST, None, page, query)
        return self.http_post(self.API_FILTERS_URL, None,
                              body=result.get_json_encode_dict_filter_for_content_based_requests())

    def list_detected_apps(self, page, filter_id):
        """
        @param $filter_id id to the filter.
        @return Json structure with the details of applications detected by the filter.
        """
        result = ExternalApiFilterRequest(ExternalApiFilterRequest.LIST_DETECTIONS_REQUEST, None, page, filter_id)
        return self.http_post(self.API_FILTERS_URL, None,
                              body=result.get_json_encode_dict_filter_for_content_based_requests())

    def list_group_detected_apps(self, page, groupName):
        """
        @param $groupName name of the group.
        @param $page A number greater or equal to 1 indicating the page of results which have to be retrieved.
        @return Json structure with the details of applications detected by the filters group.
        """
        result = ExternalApiFilterRequest(ExternalApiFilterRequest.LIST_GROUP_DETECTIONS, None, page, groupName)
        return self.http_post(self.API_FILTERS_URL, None,
                              body=result.get_json_encode_dict_filter_for_content_based_requests())

    def unsubscribe_public_filter(self, filter_id):
        """
        With this method you can subscribe to filter.
        @param $filter_id id to filter you want subscribe.
        """
        result = ExternalApiFilterRequest(ExternalApiFilterRequest.UNSUBSCRIBE_REQUEST, None, 0, filter_id)
        return self.http_post(self.API_FILTERS_URL, None,
                              body=result.get_json_encode_dict_filter_for_content_based_requests())

    def subscribe_public_filter(self, filter_id):
        """
        With this method you can unsubscribe to filter.
        @param $filter_id id to filter you want unsubscribe.
        """
        result = ExternalApiFilterRequest(ExternalApiFilterRequest.SUBSCRIBE_REQUEST, None, 0, filter_id)
        return self.http_post(self.API_FILTERS_URL, None,
                              body=result.get_json_encode_dict_filter_for_content_based_requests())

    def get_RSS_info(self, filter_id):
        """
        This method get the RSS information of a filter.
        @param $filter_id id to filter you want get RSS information.
        """
        result = ExternalApiFilterRequest(ExternalApiFilterRequest.GET_RSS_REQUEST, None, None, filter_id)
        return self.http_post(self.API_FILTERS_URL, None,
                              body=result.get_json_encode_dict_filter_for_content_based_requests())

    def get_group_RSS_info(self, groupName):
        """
        This method get the RSS information of a filters group.
        @param $groupName name of the filters group you want get RSS information.
        """
        result = ExternalApiFilterRequest(ExternalApiFilterRequest.GET_GROUP_RSS, None, None, groupName)
        return self.http_post(self.API_FILTERS_URL, None,
                              body=result.get_json_encode_dict_filter_for_content_based_requests())

    def compare_apps(self, apps, include_details):
        """
        @param $apps the key of the app you want to compare. The array of apps is limited to 10 apps.
        @param $include_details with a value of true in includeDetails you will get not only the matching fields and their values, but all the values defined for the applications.
        """
        result = ExternalApiCompareRequest(apps, include_details)
        return self.http_post(self.API_COMPARER_URL, None, body=result.get_json_encode_for_compare_apps())

    def upload_app(self, apk_file, tagName=None):
        """
        Upload app to Tacyt
        :param apk_file: path to file apk
        :return: Response
        """
        try:
            f = open(apk_file, 'rb')
            file_name = path.basename(f.name)
            file_stream = f.read()
            f.close()
            headers = dict()
            headers[Auth.FILE_HASH_HEADER_NAME] = hashlib.sha1(file_stream).hexdigest()

            return self.http_post_file(self.API_UPLOAD_URL,
                                       self.authentication_headers_with_body(self.HTTP_METHOD_POST, self.API_UPLOAD_URL,
                                                                             headers, ""), file_stream, file_name,
                                       tagName)

        except Exception as e:
            logger.exception("Error in uploading of {}".format(apk_file))
            return None



# -*- coding=utf-8 -*-

import os
from setuptools import setup

def raw_read(fname):
    with open(os.path.join(os.path.dirname(__file__), fname), 'r') as f:
        return f.read()

try:
    from pypandoc import convert_text
    read = lambda fname: convert_text(raw_read(fname), 'rst', format='md')
except ImportError:
    print("warning: pypandoc module not found, could not convert Markdown to RST")
    read = raw_read

setup(
    name='taza',
    version='1.2.1',
    author='Daniel Domínguez',
    author_email='daniel.dominguez@imdea.org',
    description= ('A set of classes and abstractions for working with Tacyt'),
    license='MIT',
    keywords='mobile tacyt android',
    url = 'https://gitlab.software.imdea.org/android/taza',
    packages = ['taza', 'taza/tacyt', 'taza/tacyt/authorization'],
    long_description = read('README.md'),
)




import os
import logging
from taza.tacyt.TacytApp import TacytApp


RESULTS_HARD_LIMIT = 1000


class APILimitReached(Exception):
    pass


class TacytException(Exception):
    def __init__(self, code, message):
        self.code = code
        self.message = message


class TooManyResultsException(Exception):
    pass


class MissingEnvironmentException(Exception):
    def __init__(self, var):
        super().__init__(f"The environment variable $var is missing")


class SearchQuery(object):
    """
    This class implements the logic for requesting apps to the search endpoint.

    To get the data of the query just iterate over the instance.

    This class doesn't make any assumptions about default values. It is recomended to 
    use the `search_apps_with_query` method in the `TazaClient` class.
    """

    def __init__(self, query, fields, grouped_results, logger, max_results, api, ignore_errors):
        """
        Initializes an instance of a query. The query is performed against tacyt each time is iterated.

        @param $query The query to be send to Tacyt
        @param $fields The fields to be requested from Tacyt
        @param $grouped_results Whenever to group the results or not
        @param $logger A logger
        @param $max_results Max number of results per page
        @param $api A TacytApp instance
        """
        self.query = query
        if not fields:
            self.fields = []
        else:
            self.fields = fields
        self.grouped_results = grouped_results
        self.__logger = logger
        self.max_results = max_results
        self.api = api
        self.ignore_errors = ignore_errors

    def __iter__(self):
        """
        Returns a generator that yields all the apps that the query returns from Tacyt.

        @return A generator of apps 
        """
        has_tail = True
        page = 1
        page_limit = RESULTS_HARD_LIMIT / self.max_results
        while has_tail:
            try:
                n_results, apps, has_tail = self.__send_query(page)
                if n_results > RESULTS_HARD_LIMIT and not self.ignore_errors:
                    raise TooManyResultsException()
                for app in apps:
                    yield app
                page += 1
                if page > page_limit:
                    if self.ignore_errors:
                        return
                    else:
                        raise TooManyResultsException()
            except TacytException as e:
                if not self.ignore_errors:
                    raise e

    def __send_query(self, page):
        """
        Will send the query requesting a certain page of the results.

        @param $page The page number of the query.
        @return The number of results, the results of the query and whenever there is more results pending.
        """
        self.__logger.debug("Sending query \"%s\" at page %d", str(self.query), page)
        result = self.api.search_apps(str(self.query),
                                      maxResults=self.max_results,
                                      numberPage=page,
                                      outfields=self.fields,
                                      grouped=self.grouped_results)
        err = result.get_error()
        if err:
            self.__logger.warning("Error #{} in page #{} from Tacyt: {}".format(err.code, page, err.message))
            if err.code == 112:
                raise APILimitReached()
            else:
                raise TacytException(err.code, err.message)
        result = result.get_data()['result']
        return (result['numResults'],                               # Number of results
                result['applications'],                             # Applications
                result['numResults'] > (self.max_results * page))   # More results or not


class TazaClient(object):
    """
    Handles the connection between the applicationa and the TacytApp class.
    It's mostly an utility class that implements common actions.

    If you come up with a query that more people could use, put it here.

    Instances of this class should be created with the factory method:

    client = TazaClient.from_credentials(APP_ID, SECRET_KEY)
    """
    
    def __init__(self, wrapped_api):
        """
        Initializes a TazaClient instance.

        @param $wrapped_api A TacytApp instance
        """
        self.api = wrapped_api
        self.__logger = logging.getLogger('TazaClient[%d]' % id(self))        

    @staticmethod
    def from_credentials(app_id, secret_key):
        """
        Factory method for creating instances of TazaClient class using the Tacyt credentials.

        @param $app_id APP_ID credential for Tacyt.
        @param $secret_key SECRET_KEY credential for Tacyt.
        @return A TazaClient instance
        """
        return TazaClient(TacytApp(app_id, secret_key))

    @staticmethod
    def from_env(app_id='APP_ID', secret_key='SECRET_KEY'):
        """
        Factory method for creating instances of TazaClient class using the credentials.
        The credentials must be stores in environment variables.

        @param $app_id The name of the env variable that holds the APP_ID.
        @param $secret_key The name of the env variable that holds the SECRET_KEY.
        @return A TazaClient instance
        """
        if app_id not in os.environ:
            raise MissingEnvironmentException(app_id)
        if secret_key not in os.environ:
            raise MissingEnvironmentException(secret_key)
        return TazaClient(TacytApp(os.environ[app_id], os.environ[secret_key]))

    def search_apps_with_query(self, query, fields=None, max_results=100, grouped_results=False, ignore_errors=False):
        """
        Will invoke the search_apps method from TacytApp automatically handling the pagination.
        Returns an instance of an iterable class.

        @param $query: The query to be sent
        @param $fields: The fields of the apps that are going to be requested
        @param $max_results: How many results per page
        @param $grouped_results: Whenever to group the results or not
        @return A SearchQuery instance
        """
        if not fields:
            fields = []
        return SearchQuery(query, fields, grouped_results, 
            self.__logger, max_results, self.api, ignore_errors)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
 This library offers an API to use Tacyt in a python environment.
 Copyright (C) 2015 Eleven Paths
'''

try:
    import simplejson as json

except ImportError:
    import json

json_encode = lambda x: json.dumps(x)
json_decode = lambda x: json.loads(x)


class ExternalApiFilterRequest():
    JSON_FIELD_OPERATION = "operation"
    JSON_FIELD_DETECTIONS = "detections"

    FIELD_FILTER = "filter"
    FIELD_FILTERS = "filters"
    FIELD_ID = "id"
    FIELD_TOTAL_COUNT = "totalCount"
    FIELD_LINK = "link"
    FIELD_USER = "user"
    FIELD_PASS = "pass"
    FILTER_MAX_SIZE = 8000
    ERROR_LIMIT_EXCEEDED = "Your filter definition exceeds the size limit of " + str(
        FILTER_MAX_SIZE) + " characters. Please, shorten the description or split the rules in multiple filters."

    CREATE_REQUEST = "CREATE"
    READ_REQUEST = "READ"
    UPDATE_REQUEST = "UPDATE"
    DELETE_REQUEST = "DELETE"
    SEARCH_PUBLIC_FILTER_REQUEST = "SEARCH_PUBLIC_FILTER"
    SUBSCRIBE_REQUEST = "SUBSCRIBE"
    UNSUBSCRIBE_REQUEST = "UNSUBSCRIBE"
    GET_RSS_REQUEST = "GET_RSS"
    LIST_DETECTIONS_REQUEST = "LIST_DETECTIONS"
    READ_GROUPS = "READ_GROUPS"
    LIST_GROUP_DETECTIONS = "LIST_GROUP_DETECTIONS"
    GET_GROUP_RSS = "GET_GROUP_RSS"

    request_type = None
    filter = None
    page = None
    content = None

    def __init__(self, request_type=None, filter=None, page=None, content=None):
        self.request_type = request_type
        self.content = content

        if page < 1 and page is not None:
            self.page = 1
        else:
            self.page = page

        self.filter = filter

    def get_json_encode_for_filter_based_requests(self):
        json_obj = dict()

        if self.request_type is not None:
            json_obj = {"requestType": self.request_type}

        if self.filter is not None:
            json_obj["filter"] = self.filter.get_json_encode()

        if (len(json.dumps(json_obj))) > self.FILTER_MAX_SIZE:
            raise Exception(self.ERROR_LIMIT_EXCEEDED)
        else:
            return json_obj

    def get_json_encode_dict_filter_for_content_based_requests(self):
        json_obj = dict()

        if self.request_type is not None:
            json_obj["requestType"] = self.request_type

        if self.content is not None:
            json_obj["content"] = self.content

        if self.page is not None:
            json_obj["page"] = self.page

        if (len(json.dumps(json_obj))) > self.FILTER_MAX_SIZE:
            raise Exception(self.ERROR_LIMIT_EXCEEDED)
        else:
            return json_obj

    def get_json_encode_string(self):

        return json_encode(self.get_json_encode_for_filter_based_requests)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

'''
 This library offers an API to use Tacyt in a python environment.
 Copyright (C) 2015 Eleven Paths
'''

import json


class Error(object):

    def __init__(self, json_data):

        self.code = json_data['code']
        self.message = json_data['message']

    def get_code(self):
        return self.code

    def get_message(self):
        return self.message

    def to_json(self):
        return {"code" : self.code, "message" : self.message}

    def __repr__(self):
        return json.dumps(self.to_json())

    def __str__(self):
        return self.__repr__()


#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''
This library offers an API to use Tacyt in a python environment.
Copyright (C) 2015 Eleven Paths
'''

from io import StringIO


class ExternalApiSearchRequest:

    grouped = None
    numberPage = 1
    maxResults = 20
    query = None
    outputFields = None

    def __init__(self, query=None, numberPage=None, maxResults=None, outputFields=None, grouped=None):

        string_build = StringIO()
        if (numberPage <= 0):
            self.numberPage = 1
        else:
            self.numberPage = numberPage


        if (maxResults <= 0):
            self.maxResults = 20

        else:
            self.maxResults = maxResults

        if (outputFields != None and len(outputFields) > 0):
            for field in outputFields:
                string_build.write(field + ",")

        self.outputFields = string_build.getvalue()
        self.grouped = grouped
        self.query = query

    def get_json_encode_for_search(self):

        json_data = dict()
        if self.query is not None:
            json_data["query"] = self.query
        if self.numberPage is not None:
            json_data["numberPage"] = self.numberPage
        if self.maxResults is not None:
            json_data["maxResults"] = self.maxResults
        if self.outputFields is not None:
            json_data["outputFields"] = self.outputFields
        if self.grouped is not None:
            json_data["grouped"] = self.grouped

        return json_data
